package cn.iocoder.yudao.module.reward.service.reward;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.reward.vo.reward.*;
import cn.iocoder.yudao.module.reward.dal.dataobject.reward.RewardDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.reward.convert.reward.RewardConvert;
import cn.iocoder.yudao.module.reward.dal.mysql.reward.RewardMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.reward.enums.ErrorCodeConstants.*;
import static com.baomidou.mybatisplus.core.toolkit.IdWorker.getId;

/**
 * 礼品 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class RewardServiceImpl extends AbstractService<RewardMapper, RewardDO> implements RewardService {

    @Resource
    private RewardMapper rewardMapper;

    @Override
    public Long create(RewardCreateReqVO createReqVO) {
        // 插入
        RewardDO rewardDO = RewardConvert.INSTANCE.convert(createReqVO);
        rewardDO.setCode(generateRewardCode());
        rewardMapper.insert(rewardDO);
        // 返回
        return rewardDO.getId();
    }

    @Override
    public void update(RewardUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateExists(updateReqVO.getId());
        // 更新
        RewardDO updateObj = RewardConvert.INSTANCE.convert(updateReqVO);
        rewardMapper.updateById(updateObj);
    }

    @Override
    public void delete(Long id) {
        // 校验存在
        this.validateExists(id);
        // 删除
        rewardMapper.deleteById(id);
    }

    private void validateExists(Long id) {
        if (rewardMapper.selectById(id) == null) {
            throw exception(_NOT_EXISTS);
        }
    }

    @Override
    public RewardDO get(Long id) {
        return rewardMapper.selectById(id);
    }

    @Override
    public List<RewardDO> getList(Collection<Long> ids) {
        return rewardMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<RewardDO> getPage(RewardQueryVO query, PageVO page) {

        LambdaQuery<RewardDO> lambdaQuery = new LambdaQuery<>();
        //礼品ID
        lambdaQuery.eqIfPresent(RewardDO::getCode, query.getCode());
        //礼品名称，模糊匹配三种语言
        if (query.getTitle() != null) {
            lambdaQuery.and(wrapper ->
                    wrapper.like(RewardDO::getTitleZh, query.getTitle())
                            .or()
                            .like(RewardDO::getTitleEn, query.getTitle())
                            .or()
                            .like(RewardDO::getTitleFr, query.getTitle())
            );
        }
        //兑换积分
        if (query.getPointsRequireSymbol() != null && query.getPointsRequire() != null) {
            lambdaQuery.gt(query.getPointsRequireSymbol() == 1, RewardDO::getPointsRequire, query.getPointsRequire())
                    .eq(query.getPointsRequireSymbol() == 2, RewardDO::getPointsRequire, query.getPointsRequire())
                    .lt(query.getPointsRequireSymbol() == 3, RewardDO::getPointsRequire, query.getPointsRequire());
        }
        //兑换网点id，领取方式，状态
        lambdaQuery.eqIfPresent(RewardDO::getNodeId, query.getNodeId())
                .eqIfPresent(RewardDO::getPickMethod, query.getPickMethod())
                .eqIfPresent(RewardDO::getStatus, query.getStatus());
        //备注，模糊匹配三种语言
        if (query.getRemark() != null) {
            lambdaQuery.and(wrapper ->
                    wrapper.like(RewardDO::getRemarkZh, query.getRemark())
                            .or()
                            .like(RewardDO::getRemarkEn, query.getRemark())
                            .or()
                            .like(RewardDO::getRemarkFr, query.getRemark())
            );
        }
        //剩余数量
        if (query.getQuantityRemainSymbol() != null && query.getQuantityRemain() != null) {
            lambdaQuery.gt(query.getQuantityRemainSymbol() == 1, RewardDO::getQuantityRemain, query.getQuantityRemain())
                    .eq(query.getQuantityRemainSymbol() == 2, RewardDO::getQuantityRemain, query.getQuantityRemain())
                    .lt(query.getQuantityRemainSymbol() == 3, RewardDO::getQuantityRemain, query.getQuantityRemain());
        }
        //创建时间
        lambdaQuery.betweenIfPresent(RewardDO::getCreateTime, query.getBeginCreateTime(), query.getEndCreateTime())
                .orderByDesc(RewardDO::getId);
        return rewardMapper.selectPage(page, lambdaQuery);
    }

    @Override
    public List<RewardDO> getList(RewardQueryVO query) {
        return rewardMapper.selectList(query);
    }

    //生成礼品ID
    private String generateRewardCode() {
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        return uuid.substring(0, 12).toUpperCase();
    }
}
